<?php

namespace SimpleCrud\Queries;

/**
 * Common function to manage LIMIT clause.
 *
 * @property \SimpleCrud\Entity $entity
 */
trait LimitTrait
{
    protected $limit;
    protected $offset;

    /**
     * Adds a LIMIT clause.
     *
     * @param int $limit
     *
     * @return self
     */
    public function limit($limit)
    {
        $this->limit = $limit;

        return $this;
    }

    /**
     * Adds an offset to the LIMIT clause.
     *
     * @param int $offset
     *
     * @return self
     */
    public function offset($offset)
    {
        $this->offset = $offset;

        return $this;
    }

    /**
     * Generate LIMIT clause.
     *
     * @return string
     */
    protected function limitToString()
    {
        if (!empty($this->limit)) {
            $query = ' LIMIT';

            if (!empty($this->offset)) {
                $query .= ' '.$this->offset.',';
            }

            return $query.' '.$this->limit;
        }

        return '';
    }
}
